@@ -10,8 +10,7 @@ import time
 import traceback
 import types
 import urllib.parse
-from configparser import SafeConfigParser
-from distutils.version import LooseVersion
+from configparser import ConfigParser
 from typing import (
     IO,
     Any,
@@ -28,22 +27,52 @@ from typing import (
 
 import distro
 import requests
+from typing_extensions import Literal
 
-__version__ = "0.8.1"
+__version__ = "0.8.2"
 
 # Ensure the Python version is supported
 assert sys.version_info >= (3, 6)
 
 logger = logging.getLogger(__name__)
 
-# Check that we have a recent enough version
-# Older versions don't provide the 'json' attribute on responses.
-assert LooseVersion(requests.__version__) >= LooseVersion("0.12.1")
 # In newer versions, the 'json' attribute is a function, not a property
 requests_json_is_function = callable(requests.Response.json)
 
 API_VERSTRING = "v1/"
 
+# An optional parameter to `move_topic` and `update_message` actions
+# See eg. https://zulip.com/api/update-message#parameter-propagate_mode
+EditPropagateMode = Literal["change_one", "change_all", "change_later"]
+
+# Generally a `reaction_type` is present whenever an emoji is specified:
+# - Optional parameters to actions: `add_reaction`, `remove_reaction`
+# - Events: "user_status", "reaction", "message", "update_message"
+# - Inside each reaction in the `reactions` field of returned message objects.
+EmojiType = Literal["realm_emoji", "unicode_emoji", "zulip_extra_emoji"]
+
+# Message flags which may be directly modified by the current user:
+# - Updated by `update_message_flags` (and for the `read` flag, also
+#   the `mark_all_as_read`, `mark_stream_as_read`, and
+#   `mark_topic_as_read` actions).
+# - User is notified of changes via `update_message_flags` events.
+# See subset of https://zulip.com/api/update-message-flags#available-flags
+ModifiableMessageFlag = Literal["read", "starred", "collapsed"]
+
+# All possible message flags.
+# - Generally present in `flags` object of returned message objects.
+# - User is notified of changes via "update_message_flags" and `update_message`
+#   events. The latter is important for clients to learn when a message is
+#   edited to mention the current user or contain an alert word.
+# See https://zulip.com/api/update-message-flags#available-flags
+MessageFlag = Literal[
+    ModifiableMessageFlag,
+    "mentioned",
+    "wildcard_mentioned",
+    "has_alert_word",
+    "historical",
+]
+
 
 class CountingBackoff:
     def __init__(
@@ -396,9 +425,9 @@ class Client:
             config_file = get_default_config_filename()
 
         if config_file is not None and os.path.exists(config_file):
-            config = SafeConfigParser()
+            config = ConfigParser()
             with open(config_file) as f:
-                config.readfp(f, config_file)
+                config.read_file(f, config_file)
             if api_key is None:
                 api_key = config.get("api", "key")
             if email is None:
@@ -652,10 +681,7 @@ class Client:
                     continue
                 else:
                     end_error_retry(False)
-                    return {
-                        "msg": f"Connection error:\n{traceback.format_exc()}",
-                        "result": "connection-error",
-                    }
+                    raise
             except requests.exceptions.ConnectionError:
                 if not self.has_connected:
                     # If we have never successfully connected to the server, don't
@@ -667,16 +693,10 @@ class Client:
                 if error_retry(""):
                     continue
                 end_error_retry(False)
-                return {
-                    "msg": f"Connection error:\n{traceback.format_exc()}",
-                    "result": "connection-error",
-                }
+                raise
             except Exception:
                 # We'll split this out into more cases as we encounter new bugs.
-                return {
-                    "msg": f"Unexpected error:\n{traceback.format_exc()}",
-                    "result": "unexpected-error",
-                }
+                raise
 
             try:
                 if requests_json_is_function:
@@ -753,16 +773,28 @@ class Client:
             if queue_id is None:
                 (queue_id, last_event_id) = do_register()
 
-            res = self.get_events(queue_id=queue_id, last_event_id=last_event_id)
+            try:
+                res = self.get_events(queue_id=queue_id, last_event_id=last_event_id)
+            except (
+                requests.exceptions.Timeout,
+                requests.exceptions.SSLError,
+                requests.exceptions.ConnectionError,
+            ):
+                if self.verbose:
+                    print(f"Connection error fetching events:\n{traceback.format_exc()}")
+                # TODO: Make this use our backoff library
+                time.sleep(1)
+                continue
+            except Exception:
+                print(f"Unexpected error:\n{traceback.format_exc()}")
+                # TODO: Make this use our backoff library
+                time.sleep(1)
+                continue
+
             if "error" in res["result"]:
                 if res["result"] == "http-error":
                     if self.verbose:
                         print("HTTP error fetching events -- probably a server restart")
-                elif res["result"] == "connection-error":
-                    if self.verbose:
-                        print(
-                            "Connection error fetching events -- probably server is temporarily down?"
-                        )
                 else:
                     if self.verbose:
                         print("Server returned error:\n{}".format(res["msg"]))
@@ -1383,12 +1415,16 @@ class Client:
         )
 
     def remove_subscriptions(
-        self, streams: Iterable[str], principals: Union[Sequence[str], Sequence[int]] = []
+        self,
+        streams: Iterable[str],
+        principals: Optional[Union[Sequence[str], Sequence[int]]] = None,
     ) -> Dict[str, Any]:
         """
         See examples/unsubscribe for example usage.
         """
-        request = dict(subscriptions=streams, principals=principals)
+        request: Dict[str, object] = dict(subscriptions=streams)
+        if principals is not None:
+            request["principals"] = principals
         return self.call_endpoint(
             url="users/me/subscriptions",
             method="DELETE",
@@ -1634,7 +1670,7 @@ class Client:
         topic: str,
         new_topic: Optional[str] = None,
         message_id: Optional[int] = None,
-        propagate_mode: str = "change_all",
+        propagate_mode: EditPropagateMode = "change_all",
         notify_old_topic: bool = True,
         notify_new_topic: bool = True,
     ) -> Dict[str, Any]:
